/**
  ******************************************************************************
  * @file    rf_driver_ll_bubble_rfip.h
  * @author  RF Application Team - EMEA
  * @brief   SPIRIT-3 Bubble RFIP module APIs
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
#ifndef RF_DRIVER_LL_BUBBLE_RFIP_H
#define RF_DRIVER_LL_BUBBLE_RFIP_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "spirit3.h"
#include "spirit3_evb_com.h"
#include "system_util.h"

#ifdef  USE_FULL_ASSERT
#include "rf_driver_assert.h"
#else
#define assert_param(expr) ((void)0U)
#endif

#define MINIMUM_BUBBLE_PAYLOAD_LEN		0x1
#define MAXIMUM_BUBBLE_PAYLOAD_LEN		0x8

#define IS_BUBBLE_PAYLOAD_LENGTH(PAYLOAD_LEN) ((PAYLOAD_LEN)>=MINIMUM_BUBBLE_PAYLOAD_LEN && \
                                             (PAYLOAD_LEN)<=MAXIMUM_BUBBLE_PAYLOAD_LEN)

#define IS_WAKEUP(WAKEUP) (((WAKEUP) == WAKEUP_BIT_SYNC) || \
					((WAKEUP) == WAKEUP_FRAME_SYNC) || \
					((WAKEUP) == WAKEUP_FRAME_COMPLETE) || \
					((WAKEUP) == WAKEUP_FRAME_VALID) )

#define IS_EDICAL(EDICAL) (((EDICAL) == ED_ICAL_VBAT_1_70_TO_2_00) || \
					((EDICAL) == ED_ICAL_VBAT_2_00_TO_2_25) || \
					((EDICAL) == ED_ICAL_VBAT_2_25_TO_2_50) || \
					((EDICAL) == ED_ICAL_VBAT_2_50_TO_2_75) || \
					((EDICAL) == ED_ICAL_VBAT_2_75_TO_3_00) || \
					((EDICAL) == ED_ICAL_VBAT_3_00_TO_3_25) || \
					((EDICAL) == ED_ICAL_VBAT_3_25_TO_3_50) || \
					((EDICAL) == ED_ICAL_VBAT_3_50_TO_3_75) )

#define IS_AGC_REF_MODE(REF_MODE) (((REF_MODE) == AGC_LOW_POWER_MODE) || \
					((REF_MODE) == AGC_HIGH_PERF) )

#define IS_AGC_REF_LVL(REF_LVL) (((REF_LVL) == AGC_REF_VBAT_1_60V) || \
					((REF_LVL) == AGC_REF_VBAT_1_65V) || \
					((REF_LVL) == AGC_REF_VBAT_1_70V) || \
					((REF_LVL) == AGC_REF_VBAT_1_75V) )

#define IS_AGC_SPUP_LVL(SPUP_LVL) (((SPUP_LVL) == AGC_SPUP_VBAT_1_80V) || \
					((SPUP_LVL) == AGC_SPUP_VBAT_1_85V) || \
					((SPUP_LVL) == AGC_SPUP_VBAT_1_90V) || \
					((SPUP_LVL) == AGC_SPUP_VBAT_1_95V) )

#define IS_LPF2_CAL(LPF2_CAL) (((LPF2_CAL) == LPF2_CAL_1KBPS_OP) || \
					((LPF2_CAL) == LPF2_CAL_2KBPS_OP) )

#define IS_AGC_HOLD_MODE(HOLD_MODE) (((HOLD_MODE) == AGC_HOLD_AFTER_PREAMBLE) || \
					((HOLD_MODE) == AGC_HOLD_AFTER_SYNC) )

#define IS_AGC_MODE(AGC_MODE) (((AGC_MODE) == AGC_MODE_MAX) || \
					((AGC_MODE) == AGC_MODE_OFF) || \
					((AGC_MODE) == AGC_MODE_ON) || \
					((AGC_MODE) == AGC_MODE_HOLD) )

#define IS_IRQ_ENABLE(IRQ_EN) (((IRQ_EN) == IRQ_BIT_SYNC_DETECTED) || \
					((IRQ_EN) == IRQ_FRAME_SYNC_COMPLETE) || \
					((IRQ_EN) == IRQ_FRAME_COMPLETE) || \
					((IRQ_EN) == IRQ_FRAME_VALID) )

#define IS_ERROR(ERROR) (((ERROR) == ERROR_NO) || \
					((ERROR) == ERROR_SYNC) || \
					((ERROR) == ERROR_FRAMING) || \
					((ERROR) == ERROR_CRC) )

/**
 * @brief  SPIRIT3 Bubble Wakeup enumeration
 */
typedef enum {
  WAKEUP_BIT_SYNC               = 0x00, /*!< the bit Sync has been detected */
  WAKEUP_FRAME_SYNC    	        = 0x01, /*!< Frame sync detected */
  WAKEUP_FRAME_COMPLETE	        = 0x02, /*!< Frame_complete detected */
  WAKEUP_FRAME_VALID            = 0x03  /*!< Frame_Valid detected (default value) */
} WakeUpLevel;


/**
 * @brief  SPIRIT3 Bubble IRQ enumeration
 */
typedef enum {
  IRQ_BIT_SYNC_DETECTED		= 0x00, /*!< Enables interrupt on BIT_SYNC_DETECTED_F flag */
  IRQ_FRAME_SYNC_COMPLETE	= 0x01, /*!< Enables interrupt on FRAME_SYNC_COMPLETE_F flag */
  IRQ_FRAME_COMPLETE		= 0x02, /*!< Enables interrupt on FRAME_COMPLETE_F flag */
  IRQ_FRAME_VALID    		= 0x03  /*!< Enables interrupt on FRAME_VALID_F flag */
} BubbleIrqEnable;

/**
 * @brief  SPIRIT3 Bubble STATUS enumeration
 */
typedef enum {
  STATUS_BIT_SYNC_DETECTED	= 0x00, /*!< status BIT_SYNC_DETECTED*/
  STATUS_FRAME_SYNC_COMPLETE 	= 0x01, /*!< status FRAME_SYNC_COMPLETE */
  STATUS_FRAME_COMPLETE		= 0x02, /*!< status FRAME_COMPLETE */
  STATUS_FRAME_VALID    	= 0x03  /*!< status FRAME_VALID */
} BubbleSatus;

/**
 * @brief  SPIRIT3 Bubble ERROR enumeration
 */
typedef enum {
  ERROR_NO		= 0x00, /*!< NO ERROR */
  ERROR_SYNC	= 0x01, /*!< error in SYNC word */
  ERROR_FRAMING	= 0x02, /*!< FRAMING error */
  ERROR_CRC    	= 0x03  /*!< CRC error */
} BubbleError;

/**
 * @brief  SPIRIT3 Bubble ED_ICAL enumeration
 */
typedef enum {
  ED_ICAL_VBAT_1_70_TO_2_00	= 0x00, /*!< VBAT=1.7 to 2.0V (default value) */
  ED_ICAL_VBAT_2_00_TO_2_25	= 0x01, /*!< VBAT=2.0 to 2.25V */
  ED_ICAL_VBAT_2_25_TO_2_50	= 0x02, /*!< VBAT=2.25 to 2.5V */
  ED_ICAL_VBAT_2_50_TO_2_75	= 0x03, /*!< VBAT=2.5 to 2.75V */
  ED_ICAL_VBAT_2_75_TO_3_00	= 0x04, /*!< VBAT=2.75 to 3.0V */
  ED_ICAL_VBAT_3_00_TO_3_25	= 0x05, /*!< VBAT=3.0 to 3.25V */
  ED_ICAL_VBAT_3_25_TO_3_50	= 0x06, /*!< VBAT=3.25 to 3.5V */
  ED_ICAL_VBAT_3_50_TO_3_75	= 0x07  /*!< VBAT=3.5 to 3.75V */
} EdIcal;

/**
 * @brief  SPIRIT3 Bubble AGC_REF_MODE enumeration
 */
typedef enum {
  AGC_LOW_POWER_MODE	= 0x00, /*!< Low Power Mode (default) */
  AGC_HIGH_PERF		= 0x01  /*!< HIgh Performance Mode */
} AgcRefMode;

/**
 * @brief  SPIRIT3 Bubble AGC_REF_LVL enumeration
 */
typedef enum {
  AGC_REF_VBAT_1_60V	= 0x00, /*!< VBAT-1.6V (default) */
  AGC_REF_VBAT_1_65V   	= 0x01, /*!< VBAT-1.65V */
  AGC_REF_VBAT_1_70V	= 0x02, /*!< VBAT-1.7V */
  AGC_REF_VBAT_1_75V 	= 0x03  /*!< VBAT-1.75V */
} AgcRefLevel;

/**
 * @brief  SPIRIT3 Bubble AGC_SPUP_LVL enumeration
 */
typedef enum {
  AGC_SPUP_VBAT_1_80V	= 0x00, /*!< VBAT-1.8V */
  AGC_SPUP_VBAT_1_85V	= 0x01, /*!< VBAT-1.85V */
  AGC_SPUP_VBAT_1_90V	= 0x02, /*!< VBAT-1.9V */
  AGC_SPUP_VBAT_1_95V	= 0x03  /*!< VBAT-1.95V */
} AgcSpupLevel;

/**
 * @brief  SPIRIT3 Bubble LPF2_CAL enumeration
 */
typedef enum {
  LPF2_CAL_1KBPS_OP	= 0x00, /*!< 1kbps operation (default) */
  LPF2_CAL_2KBPS_OP    	= 0x01  /*!< 2kbps operation */
} Lpf2Cal;

/**
 * @brief  SPIRIT3 Bubble AGC Hold Mode enumeration
 */
typedef enum {
  AGC_HOLD_AFTER_PREAMBLE	= 0x00, /*!< AGC Hold after preamble detection (default) */
  AGC_HOLD_AFTER_SYNC		= 0x01  /*!< AGC Hold after Sync detection */
} AgcHoldType;

/**
 * @brief  SPIRIT3 Bubble AGC Hold Mode enumeration
 */
typedef enum {
  AGC_MODE_MAX 			= 0x00, /*!< AGC Mode MAX (default) */
  AGC_MODE_OFF    		= 0x01, /*!< AGC Mode OFF */
  AGC_MODE_ON     		= 0x02, /*!< AGC Mode ON */
  AGC_MODE_HOLD    		= 0x03  /*!< AGC HOLD: the AGC is ON and its value is frozen during the bubble frame detection*/
} AgcMode;

typedef enum {
  AGC_RESET_MODE_AFTER_FRAME    = 0x00, /*!< reset the AGC after the complete frame detection */
  AGC_RESET_MODE_NEVER          = 0x01  /*!< the AGC is never reset */
} AgcResetMode;

/**
 * @brief  SPIRIT3 Bubble STATUS enumeration
 */
typedef enum {
  BIT_SYNC_DETECTED_F   = 0x00, /*!< Preamble (= bit sync) has been detected, the content of the PAYLOAD_X registers is not yet	valid */
  FRAME_SYNC_COMPLETE_F = 0x01, /*!< Frame Sync has been detected, the content of the PAYLOAD_X registers is not yet valid */
  FRAME_COMPLETE_F	= 0x02, /*!< Frame (payload + CRC) received, the content of the PAYLOAD_X registers is valid */
  FRAME_VALID_F		= 0x03, /*!< Frame (payload + CRC) received without error (the CRC has been checked and is matching with the received CRC).*/
  NO_STATUS			= 0x04  /*!< Frame (payload + CRC) received without error (the CRC has been checked and is matching with the received CRC).*/
} BubbleStatus;


/**
 * @brief  SPIRIT3 Bubble Frame Init structure definition
 */
typedef struct {
  uint8_t		SlowClkCyclePerBitCnt;   /*!< The number of expected slow clock cycle per each manchester coded bit (default 16) */
  uint8_t  		PayloadLength; 		 /*!<  The number of data Bytes in the payload (decoded).
 						Only the values 1 (8bits) to 8 (64bits) included are supported, other values are not supported
						and shall not be used.
 						Default value is 7 bytes / 56 bits */
  uint8_t		SyncThr;          	/*!< Detection threshold when receiving the Frame sync (Manchester encoded). The frame sync
						detection uses a correlator to detect the address.
						- If FRAME_CONFIG0.SYNC_LENGTH=0, recommended threshold is 16 (0x10)
						- If FRAME_CONFIG0.SYNC_LENGTH=1, recommended threshold is 32 (0x20)
 						Default value is 16 (0x10) */
  uint8_t         SyncLength;           	/*!< Frame sync pattern length:
						- If the input signal sync_len=0 (default), a 8-bits frame sync is used, the Low word
						(FRAME_CONFIG1.FRAME_SYNC_PATTERN_L) is describing the frame sync pattern
						value (16 bits after encoding)
						- If the input signal sync_len=1, a 16-bits frame sync is used, both Low
						(FRAME_SYNC_CONFIG.FRAME_SYNC_PATTERN_L) and High
						(FRAME_SYNC_CONFIG.FRAME_SYNC_PATTERN_H) words are storing the pattern
						value (32 bits after encoding) */
  uint8_t         PreambleThrCnt;         /*!< The number of transitions for preamble detection when receiving the Manchester encoded
						preamble.
 						Default value is 18 (0x12)*/
  FunctionalState	PreambleEnable;	        /*!< Preamble detection enable.
						- If PREAMBLE_ENABLE = 1 (default value) the sync detection starts after the preamble
						found,
						- If PREAMBLE_ENABLE = 0, the sync detection starts when the bubble_rx is enabled*/
  uint8_t	      FrameSyncCntTimeout;	/*!< The timeout in Manchester encoded bits for the Frame Sync.
						It represents the number of samples after which a sync_error is raised if the frame sync is not
						detected.
						The default value is 70 (0x46)*/
  uint16_t        FrameSyncPattenHigh;	/*!<  The value of the frame sync pattern, High word, Manchester encoded, used only when the
						frame sync length is 32 bits (default 0x0000)*/
  uint16_t        FrameSyncPatternLow;	/*!< The value of the frame sync pattern, Low word, Manchester encoded, used both when the
						frame sync length is 16 bits and 32 bits (default 0x9696 which represent a frame sync value of
						0x99)*/
} SBubbleFrameInit;

/**
 * @brief  SPIRIT3 Bubble RF Config structure definition
 */
typedef struct {
  EdIcal          	EnergyDetectorIcal;     /*!< Current versus VBAT calibration for ED*/
  AgcRefMode            AgcReferenceMode; 	/*!< AGC reference mode REF_LEVEL = 200mV SPUP_LEVEL=0V*/
  AgcRefLevel           AgcReferenceLevel; 	/*!< Select Low Power reference level*/
  AgcSpupLevel     	AgcSpeedUpLevel;        /*!< Select Low Power Speed Up level*/
  FunctionalState		AgcSpeedUpEnable;		/*!< Agc Speed Up enable */
  uint8_t			LPF1_Cal;			/*!< Low Pass filter calibration at Detector output*/
  Lpf2Cal			LPF2_Cal;			/*!< LPF calibration for SC filter*/
  uint8_t			ClockDivider;		/*!< Calibrate the 4kHz clock (programmable divider)*/
  FunctionalState		EnergyDetectorSwitch;	/*!< Energy Detector as input switch (test mode) deactivated by default*/
  AgcResetMode		AgcResetMode;		/*!< The AGC reset behavior when the AGC is working in ON or HOLD mode
 									0x0: reset the AGC after the complete frame detection (default)
 									0x1: the AGC is never reset*/
  AgcHoldType		AgcHoldMode;		/*!< The behavior when the AGC is ON and is working in HOLD mode
 									0x0: hold after the preamble detection (default)
 									0x1: hold after the sync detection*/
  AgcMode			AgcMode;			/*!< Define the working mode of the AGC:
 									0x0: AGC MAX (default value) : the AGC is ON and is always operating at maximum value
 									0x1: AGC OFF: the AGC is OFF
 									0x2: AGC ON : the AGC is ON
 									0x3: AGC HOLD: the AGC is ON and its value is frozen during the bubble frame detection,
									depending of the AGC_HOLD_MODE configuration after a preamble or after the frame sync.*/
} SBubbleRFConfig;

/**
 * @brief  SPIRIT3 Bubble RF version structure definition
 */
typedef struct {
  uint8_t         revision;	/*!< Revision of the RFIP (to be used for metal fixes)*/
  uint8_t  		version;	/*!< Version of the RFIP (to be used for cut upgrades)*/
  uint8_t       	product;	/*!< Used for major upgrades (new protocols support / new features)*/
} SBubbleVersion;

/**
 * @brief  SPIRIT3 BUBBLE_RFIP public interface
 */
SBubbleVersion BubbleGetVersion(void);


void BubbleRFConfigInit(SBubbleRFConfig* pxSBubbleRFConfigStruct);
SBubbleRFConfig BubbleGetRFConfig(void);

void BubbleFrameInit(SBubbleFrameInit* pxSBubbleFrameInitStruct);
SBubbleFrameInit BubbleGetFrameInfo(void);

void BubbleIrqConfig(BubbleIrqEnable IrqEnable);

BubbleStatus BubbleGetStatus(void);
void BubbleClearStatus(void);

BubbleError BubbleReadError(void);
void BubbleClearErrorStatus(void);

void BubbleSetSlowClkCyclePerBitCnt(uint8_t ClkCyclePerBit);
uint8_t BubbleGetSlowClkCyclePerBitCnt(void);

void BubbleSetPayloadLength(uint8_t length);
uint8_t BubbleGetPayloadLength(void);

void BubbleSetSyncThrCnt(uint8_t SyncThrCnt);
uint8_t BubbleGetSyncThrCnt(void);

void BubbleSetSyncLength(uint8_t SyncLen);
uint8_t BubbleGetSyncLength(void);

void BubbleSetPreambleThrCnt(uint8_t cnt);
uint8_t BubbleGetPreambleThrCnt(void);

void BubbleSetPreambleEnable(FunctionalState state);
FunctionalState BubbleGetPreambleEnable(void);

void BubbleSetFrameSyncCntTimeout(uint8_t timeout);
uint8_t BubbleGetFrameSyncCntTimeout(void);

void BubbleSetKp(uint8_t v_kp);
uint8_t BubbleGetKp(void);

void BubbleSetKi(uint8_t v_ki);
uint8_t BubbleGetKi(void);

void BubbleSetSync(uint32_t sync);
uint32_t BubbleGetSync(void);
uint16_t BubbleGetSyncPatternHigh(void);
uint16_t BubbleGetSyncPatternLow(void);

void BubbleSetWakeUpLevel(WakeUpLevel xWakeUpLvl);
WakeUpLevel BubbleGetWakeUpLevel(void);

void BubbleSetState(FunctionalState state);
FunctionalState BubbleGetState(void);

void BubbleSetEdIcal(EdIcal value);
EdIcal BubbleGetEdIcal(void);

void BubbleSetAgcRefMode(AgcRefMode mode);
AgcRefMode BubbleGetAgcRefMode(void);

void BubbleSetAgcRefLvl(AgcRefLevel level);
AgcRefLevel BubbleGetAgcRefLvl(void);

void BubbleSetAgcSpupLvl(AgcSpupLevel level);
AgcSpupLevel BubbleGetAgcSpupLvl(void);

void BubbleAgcSpupEnable(FunctionalState state);
FunctionalState BubbleAgcSpupGetState(void);

void BubbleSetLpf1Cal(uint8_t value);
uint8_t BubbleGetLpf1Cal(void);

void BubbleSetLpf2Cal(Lpf2Cal value);
Lpf2Cal BubbleGetLpf2Cal(void);

void BubbleSetClkDiv(uint8_t div);
uint8_t BubbleGetClkDiv(void);

void BubbleSetEdSwitch(FunctionalState state);
FunctionalState BubbleGetEdSwitch(void);

void BubbleSetAgcResetMode(AgcResetMode state);
AgcResetMode BubbleGetAgcResetMode(void);

void BubbleSetAgcHoldMode(AgcHoldType mode);
AgcHoldType BubbleGetAgcHoldMode(void);

void BubbleSetAgcMode(AgcMode mode);
AgcMode BubbleGetAgcMode(void);

void BubbleGetPayload(uint8_t* buff);


#ifdef __cplusplus
}
#endif


#endif
